from PyQt5.QtWidgets import *
from krita import *
import numpy as np
import tensorflow as tf
from PIL import Image
# from matplotlib import pyplot as plt
# plt.ion()

K = Krita.instance()

def krita2np(node,x,y,w,h):
	n = K.activeDocument()
	# n = K.activeDocument().activeNode()
	return np.array(Image.frombytes("RGBA", (w,h), n.pixelData(x,y,w,h)))

def setpix_qi(im,x,y,w=None,h=None,refresh=True):
	w = w if not w==None else im.width()
	h = h if not h==None else im.height()
	ptr = im.bits()
	ptr.setsize(im.byteCount())
	document = K.activeDocument()
	document.activeNode().setPixelData(QByteArray(ptr.asstring()),x,y,w,h)
	if refresh: document.refreshProjection()

def setpix_np(im,x,y,w=None,h=None,refresh=True):
	w = w if not w==None else im.shape[0]
	h = h if not h==None else im.shape[1]
	# im = QImage(im, im.shape[1], im.shape[0], im.shape[1] * im.shape[2], QImage.Format_RGB888)
	# TODO : this implementation has horrible performance
	qim= QImage(w, h, QImage.Format_RGB32)
	for xx in range(w):
		for yy in range(h):
			qim.setPixelColor(xx,yy, QColor(*im[xx,yy,:]))
	setpix_qi(qim,x,y,refresh=refresh)

class NNRenderDocker(DockWidget):
	def draw_model(self):
		# self.model = self.model if not self.model==None else tf.keras.models.load_model('/home/doek/git/digital_twin/Data/digital_twin_ref.h5')
		if self.model==None: self.model=tf.keras.models.load_model(self.UI_path.text())
		input_w = 128
		input_h = 128
		# x = tf.random.normal((1,128,128,1))
		x = krita2np(K.activeDocument().activeNode(),0,0,input_w,input_h)
		# x = (1-x[:,:,0:1]) * x[:,:,3:4]
		# x = x[:,:,3:4] # read from alpha
		x = x[:,:,0:1] # read from red channel
		x = np.expand_dims(x, axis=0)/255 # to float
		if self.UI_flip.isChecked(): x = 1 - x
		y = np.array(self.model(x))[0,:,:,:]   # eval with model
		im = np.stack((y*(y>0),0*y,np.abs(y)*(y<0)),axis=2)[:,:,:,0]
		# im = im*4
		im = im * (2**(self.UI_exposure.value()/10))
		im = im - (im<0)*(im-0) # if below 0 set to 0
		im = im - (im>1)*(im-1) # if above 1 set to 1
		im = (255*im).astype(np.uint8)
		setpix_np(im,128,0)

	def __init__(self):
		super().__init__()
		self.setWindowTitle("NN render")
		self.model = None

		self.UI_widget = QWidget(self)
		self.UI_layout = QVBoxLayout()

		# self.UI_path = QPlainTextEdit()
		self.UI_path = QLineEdit()
		self.UI_path.setPlaceholderText("path/to/neural-net-model.h5")
		self.UI_path.setText('/home/doek/digital_twin_ref.h5')
		# def reload_model(): self.model = tf.keras.models.load_model(self.UI_path.text())
		# self.UI_path.textChanged.connect(reload_model)
		self.UI_layout.addWidget(QLabel("h5:"))
		self.UI_layout.addWidget(self.UI_path)

		self.UI_enable=QCheckBox()
		# self.UI_layout.addWidget(self.UI_enable)
		self.UI_enable_widget = QWidget()
		self.UI_enable_container = QHBoxLayout()
		self.UI_enable_container.addWidget(QLabel("live"))
		self.UI_enable_container.addWidget(self.UI_enable)
		self.UI_enable_widget.setLayout(self.UI_enable_container)
		self.UI_layout.addWidget(self.UI_enable_widget)

		self.UI_flip=QCheckBox()
		# self.UI_layout.addWidget(self.UI_enable)
		self.UI_flip_widget = QWidget()
		self.UI_flip_container = QHBoxLayout()
		self.UI_flip_container.addWidget(QLabel("flip"))
		self.UI_flip_container.addWidget(self.UI_flip)
		self.UI_flip_widget.setLayout(self.UI_flip_container)
		self.UI_layout.addWidget(self.UI_flip_widget)

		# if self.UI_enable.isChecked(): self.draw_model()

		self.UI_render = QPushButton("render", self.UI_widget)
		self.UI_render.clicked.connect(self.draw_model)
		self.UI_layout.addWidget(self.UI_render)

		# self.UI_exposure = QSlider()
		self.UI_exposure = QSlider(Qt.Horizontal)
		self.UI_exposure.setMinimum(-40)
		self.UI_exposure.setMaximum(40)
		self.UI_exposure.setValue(20)
		# self.UI_exposure.setTickPosition(QSlider.TicksBelow)
		self.UI_exposure.setTickInterval(1)
		self.UI_layout.addWidget(self.UI_exposure)

		# self.UI_tabs = QTabWidget()
		# self.UI_tabs.addTab(self.some_widget, "tab1")
		# self.UI_tabs.addTab(self.some_widget, "tab2")

		# self.UI_layout.addWidget(self.UI_tabs)
		# self.UI_layout.addWidget(self.container)
		self.UI_layout.addStretch()
		self.UI_widget.setLayout(self.UI_layout)
		self.setWidget(self.UI_widget)

		def loop():
			if self.UI_enable.isChecked(): self.draw_model()
			QTimer.singleShot(200, loop)
		loop()

		# self.test = QSpinBox()
		# self.test = QDoubleSpinBox()
		# self.test.setMinimum(1)
		# self.test.setMaximum(250)

		# self.txt2img_widget = QWidget()
		# self.txt2img_widget.setLayout(self.txt2img_layout)

	def canvasChanged(self, canvas):
		pass

# from krita import *
# from .nnrender import NNRenderDocker

Krita.instance().addDockWidgetFactory(
	DockWidgetFactory(
		'nnrender',
		DockWidgetFactoryBase.DockRight,
		NNRenderDocker
	)
)
